@@ -1,13 +1,17 @@
-import inspect
 import re
+import copy
+import yaml
+import flask
+import random
+import inspect
+import requests
 import traceback
 from functools import wraps
-from typing import Literal, Optional
 from uuid import uuid4 as uuid
-
-import flask
-
 from pydatafront.app import app
+from urllib.parse import urlparse
+from typing import Literal, Optional
+
 
 __supported_basic_types = ["int", "float", "str", "bool"]
 __supported_types = __supported_basic_types + ["dict", "list"]
@@ -19,7 +23,35 @@ __supported_basic_types_dict = {
 }
 __decorated_functions_list = list()
 __wrapper_enabled = False
+__default_theme = {}
+__themes = {}
+__parsed_themes = {}
+__basic_widgets = ["slider", "input", "textField", "switch", "button", "checkbox"]
 
+__theme_style_sugar_dict = {
+    "fontColor": {
+        "root": {
+            "color": "${value}"
+        }
+    }
+}
+
+def dict_replace(original_dict: dict, original: str, new: any):
+    if isinstance(original_dict, dict):
+        return {
+            key: dict_replace(value, original, new)
+                for key, value in original_dict.items()
+        }
+
+    # str
+    if original_dict == original:
+        return new
+    else:
+        return original_dict
+
+def get_full_style_from_sugar(key: str, value: any):
+    sugar_info = __theme_style_sugar_dict[key]
+    return dict_replace(sugar_info, "${value}", value)
 
 def enable_wrapper():
     global __wrapper_enabled
@@ -33,6 +65,108 @@ def enable_wrapper():
             }
 
 
+def get_mui_theme(theme, colors):
+    mui_theme = {
+        "components": {},
+        "palette": {}
+    }
+    temp_colors = {}
+    if colors:
+        for color in colors.keys():
+            if color == "mode":
+                mui_theme["palette"][color] = colors[color]
+            elif isinstance(colors[color], str):
+                mui_theme["palette"][color] = {"main": colors[color]}
+            else:
+                mui_theme["palette"][color] = colors[color]
+    for widget_name in theme.keys():
+        widget_mui_name = "Mui" + widget_name[0].upper() + widget_name[1::]
+        mui_theme["components"][widget_mui_name] = {
+            "defaultProps": {},
+            "styleOverrides": {}
+        }
+        for prop_name in theme[widget_name].keys():
+            if prop_name == "style":
+                mui_theme["components"][widget_mui_name]["styleOverrides"].update(theme[widget_name][prop_name])
+            elif prop_name == "color":
+                if not theme[widget_name][prop_name] in mui_theme["palette"]:
+                    if theme[widget_name][prop_name] in temp_colors.keys():
+                        mui_theme["components"][widget_mui_name]["defaultProps"][prop_name] = \
+                            temp_colors[theme[widget_name][prop_name]]
+                    else:
+                        color_name = "temp_" + "".join(["abcdef0123456789"[random.randint(0, 15)] for _ in range(8)])
+                        mui_theme["palette"][color_name] = {"main": theme[widget_name][prop_name]}
+                        mui_theme["components"][widget_mui_name]["defaultProps"][prop_name] = color_name
+                        temp_colors[theme[widget_name][prop_name]] = color_name
+                true_color = mui_theme["palette"][color_name]["main"]
+                styleOverride = {}
+                if widget_name == "input":
+                    styleOverride = {
+                        "underline": {
+                            "&:before": {
+                                "borderColor": true_color
+                            },
+                            "&&:hover::before": {
+                                "borderColor": true_color
+                            }
+                        }
+                    }
+                if widget_name == "textField":
+                    styleOverride = {
+                        "root": {
+                            "& fieldset": {
+                                "borderColor": true_color
+                            },
+                            "&&:hover fieldset": {
+                                "border": "2px solid", # Hmmm
+                                "borderColor": true_color
+                            }
+                        }
+                    }
+                if styleOverride != {}:
+                    mui_theme["components"][widget_mui_name]["styleOverrides"].update(styleOverride)
+            elif prop_name in __theme_style_sugar_dict.keys():
+                mui_theme["components"][widget_mui_name]["styleOverrides"].update(
+                    get_full_style_from_sugar(prop_name, theme[widget_name][prop_name])
+                )
+            else:
+                mui_theme["components"][widget_mui_name]["defaultProps"][prop_name] = theme[widget_name][prop_name]
+
+
+    return mui_theme
+
+def parse_theme(theme):
+    type_names = []
+    type_widget_dict = {}
+    widget_style = {}
+    custom_palette = {}
+    if "types" in theme:
+        for type_name in theme["types"]:
+            widget_name = theme["types"][type_name]
+            if "," in type_name:
+                common_types_names = [name.strip() for name in type_name.split(",")]
+                type_names += common_types_names
+                for name in common_types_names:
+                    type_widget_dict[name] = widget_name
+            else:
+                type_names.append(type_name)
+                type_widget_dict[type_name] = widget_name
+    if "styles" in theme:
+        if "basic" in theme["styles"]:
+            for basic_widget_name in __basic_widgets:
+                widget_style[basic_widget_name] = copy.deepcopy(theme["styles"]["basic"])
+            del theme["styles"]["basic"]
+        for widget_name in theme["styles"].keys():
+            if widget_name in widget_style:
+                widget_style[widget_name].update(theme["styles"][widget_name])
+            else:
+                widget_style[widget_name] = theme["styles"][widget_name]
+    if "colors" in theme:
+        for color_name in theme["colors"].keys():
+            custom_palette[color_name] = theme["colors"][color_name]
+    mui_theme = get_mui_theme(widget_style, custom_palette)
+    return type_names, type_widget_dict, widget_style, custom_palette, mui_theme
+
 def get_type_dict(annotation):
     if isinstance(annotation, object):  # is class
         annotation_type_class_name = getattr(type(annotation), "__name__")
@@ -99,7 +233,7 @@ def get_type_dict(annotation):
         }
 
 
-def get_type_widget_prop(function_arg_type_name, index, function_arg_widget):
+def get_type_widget_prop(function_arg_type_name, index, function_arg_widget, widget_type):
     # Basic and List only
     if isinstance(function_arg_widget, str):
         widget = function_arg_widget
@@ -110,6 +244,8 @@ def get_type_widget_prop(function_arg_type_name, index, function_arg_widget):
             widget = function_arg_widget[index]
     else:
         widget = ""
+    if function_arg_type_name in widget_type:
+        widget = widget_type[function_arg_type_name]
     if function_arg_type_name in __supported_basic_types:
         return {
             "type": __supported_basic_types_dict[function_arg_type_name],
@@ -130,7 +266,7 @@ def get_type_widget_prop(function_arg_type_name, index, function_arg_widget):
             return {
                 "type": "array",
                 "widget": widget,
-                "items": get_type_widget_prop(content_type, index + 1, function_arg_widget)
+                "items": get_type_widget_prop(content_type, index + 1, function_arg_widget, widget_type)
             }
         elif function_arg_type_name == "typing.Dict":
             return {
@@ -150,12 +286,58 @@ def get_type_widget_prop(function_arg_type_name, index, function_arg_widget):
             }
 
 
+def is_valid_uri(uri: str) -> bool:
+    try:
+        result = urlparse(uri)
+        return all([result.scheme, result.netloc, result.path])
+    except:
+        return False
+
+
+def get_theme(path: str):
+    if not path:
+        return __default_theme
+    if is_valid_uri(path):
+        return yaml.load(requests.get(path).content, yaml.FullLoader)
+    else:
+        if path in __themes:
+            return __themes[path]
+        with open(path, "r", encoding = "utf-8") as f:
+            return yaml.load(f.read(), yaml.FullLoader)
+
+
+def set_global_theme(path: str) :
+    global __default_theme, __parsed_themes
+    __default_theme = get_theme(path)
+    __parsed_themes["__default"] = parse_theme(__default_theme)
+
+
+def import_theme(path: str, name: str):
+    global __themes
+    __themes[name] = get_theme(path)
+
+
 def textea_export(path: Optional[str] = None, description: Optional[str] = "",
-                  destination: Literal["column", "row", "sheet", None] = None, **decorator_kwargs):
+                  destination: Literal["column", "row", "sheet", None] = None, theme: Optional[str] = "",
+                  **decorator_kwargs):
+    global __parsed_themes
     def decorator(function: callable):
         if __wrapper_enabled:
             id: str = str(uuid())
             function_name = getattr(function, "__name__")  # function name as id to retrieve function info
+            function_theme = get_theme(theme)
+
+            try:
+                if theme == "":
+                    parsed_theme = __parsed_themes["__default"]
+                else:
+                    if theme in __parsed_themes:
+                        parsed_theme = __parsed_themes[theme]
+                    else:
+                        parsed_theme = parse_theme(function_theme)
+                        __parsed_themes[theme] = parsed_theme
+            except:
+                parsed_theme = [], {}, {}, {}, {}
 
             if path is None:
                 endpoint = function_name
@@ -241,7 +423,12 @@ def textea_export(path: Optional[str] = None, description: Optional[str] = "",
                     widget = decorated_params[function_arg_name]["widget"]
                 else:
                     widget = ""
-                json_schema_props[function_arg_name] = get_type_widget_prop(function_arg_type_dict["type"], 0, widget)
+                json_schema_props[function_arg_name] = get_type_widget_prop(
+                    function_arg_type_dict["type"],
+                    0,
+                    widget,
+                    parsed_theme[1]
+                )
                 if "whitelist" in decorated_params[function_arg_name].keys():
                     json_schema_props[function_arg_name]["whitelist"] = decorated_params[function_arg_name]["whitelist"]
                 elif "example" in decorated_params[function_arg_name].keys():
@@ -257,13 +444,14 @@ def textea_export(path: Optional[str] = None, description: Optional[str] = "",
                     else:
                         cell_type = "object"
                     json_schema_props[function_arg_name]["items"] = \
-                        get_type_widget_prop(function_arg_type_dict["type"], 0, widget[1:])
+                        get_type_widget_prop(function_arg_type_dict["type"], 0, widget[1:], {})
                     json_schema_props[function_arg_name]["type"] = "array"
 
             decorated_function = {
                 "id": id,
                 "name": function_name,
                 "params": decorated_params,
+                "theme": parsed_theme[4],
                 "return_type": return_type_parsed,
                 "description": description,
                 "schema": {
